import { __decorate, __param } from 'tslib';
import { EventEmitter, Optional, Self, KeyValueDiffers, NgZone, HostBinding, Input, Output, ViewChild, Component, ChangeDetectionStrategy, ViewEncapsulation, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { MatFormFieldControl } from '@angular/material';
import { Subject } from 'rxjs';

var MatCodemirrorComponent_1;
let MatCodemirrorComponent = MatCodemirrorComponent_1 = class MatCodemirrorComponent {
    constructor(ngControl, // mat
    _differs, _ngZone) {
        this.ngControl = ngControl;
        this._differs = _differs;
        this._ngZone = _ngZone;
        this.stateChanges = new Subject();
        this.id = `mat-codemirror-component-${MatCodemirrorComponent_1.materialId++}`;
        this.describedBy = '';
        this.onChange = (_) => null;
        this.onTouched = () => null;
        this._required = false;
        this._focused = false;
        this._disabled = false;
        this._value = '';
        this.ngxMatCodemirror = true;
        /* class applied to the created textarea */
        this.className = '';
        /* name applied to the created textarea */
        this.name = 'codemirror';
        /* autofocus setting applied to the created textarea */
        this.autoFocus = false;
        /* preserve previous scroll position after updating value */
        this.preserveScrollPosition = false;
        /* called when the text cursor is moved */
        this.cursorActivity = new EventEmitter();
        /* called when the editor is focused or loses focus */
        this.focusChange = new EventEmitter();
        /* called when the editor is scrolled */
        this.scroll = new EventEmitter();
        if (this.ngControl != null) {
            this.ngControl.valueAccessor = this;
        }
    }
    get shouldLabelFloat() {
        return this.focused || !this.empty;
    }
    setDescribedByIds(ids) {
        this.describedBy = ids.join(' ');
    }
    onContainerClick(event) { }
    get required() {
        return this._required;
    }
    set required(req) {
        this._required = coerceBooleanProperty(req);
        this.stateChanges.next();
    }
    get errorState() {
        return this.ngControl && !this.ngControl.pristine && !this.ngControl.valid;
    }
    get placeholder() {
        return this._placeholder;
    }
    set placeholder(plh) {
        this._placeholder = plh;
        this.stateChanges.next();
    }
    get empty() {
        return !this.value;
    }
    get value() {
        return this._value;
    }
    set value(value) {
        this._value = value;
        this.updateLineNumbers();
        this.stateChanges.next();
    }
    get disabled() {
        return this._disabled;
    }
    set disabled(dis) {
        this._disabled = coerceBooleanProperty(dis);
        this.stateChanges.next();
    }
    get focused() {
        return this._focused;
    }
    set focused(val) {
        this._focused = val;
        this.stateChanges.next();
    }
    /**
     * set options for codemirror
     * @link http://codemirror.net/doc/manual.html#config
     */
    set options(value) {
        this._options = value;
        if (!this._differ && value) {
            this._differ = this._differs.find(value).create();
        }
    }
    ngAfterViewInit() {
        if (!this.ref) {
            return;
        }
        // in order to allow for universal rendering, we import Codemirror runtime with `require` to prevent node errors
        const { fromTextArea } = require('codemirror');
        this.codeMirror = fromTextArea(this.ref.nativeElement, this._options);
        this.updateLineNumbers();
        this._ngZone.runOutsideAngular(() => {
            this.codeMirror.on('change', this.codemirrorValueChanged.bind(this));
            this.codeMirror.on('cursorActivity', this.cursorActive.bind(this));
            this.codeMirror.on('focus', this.focusChanged.bind(this, true));
            this.codeMirror.on('blur', this.focusChanged.bind(this, false));
            this.codeMirror.on('scroll', this.scrollChanged.bind(this));
        });
        this.codeMirror.setValue(this.value);
    }
    ngDoCheck() {
        if (this._differ) {
            // check options have not changed
            const changes = this._differ.diff(this._options);
            if (changes) {
                changes.forEachChangedItem(option => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachAddedItem(option => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachRemovedItem(option => this.setOptionIfChanged(option.key, option.currentValue));
            }
        }
    }
    ngOnDestroy() {
        // is there a lighter-weight way to remove the cm instance?
        if (this.codeMirror) {
            this.codeMirror.toTextArea();
        }
        this.stateChanges.complete();
    }
    codemirrorValueChanged(cm, change) {
        if (change.origin !== 'setValue') {
            this.value = cm.getValue();
            this.writeValue(cm.getValue());
        }
    }
    setOptionIfChanged(optionName, newValue) {
        if (!this.codeMirror) {
            return;
        }
        this.codeMirror.setOption(optionName, newValue);
    }
    focusChanged(focused) {
        this.onTouched();
        this.focused = focused;
        this.focusChange.emit(focused);
    }
    scrollChanged(cm) {
        this.scroll.emit(cm.getScrollInfo());
    }
    cursorActive(cm) {
        this.cursorActivity.emit(cm);
    }
    writeValue(value) {
        if (!this.codeMirror) {
            if (value) {
                this.value = value;
            }
            return;
        }
        if (value == null) {
            value = '';
        }
        if (value !== this.codeMirror.getValue() &&
            this.normalizeLineEndings(this.codeMirror.getValue()) !==
                this.normalizeLineEndings(value)) {
            this.value = value;
            if (this.preserveScrollPosition) {
                const prevScrollPosition = this.codeMirror.getScrollInfo();
                this.codeMirror.setValue(this.value);
                this.codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
                return;
            }
            this.codeMirror.setValue(this.value);
            // Don't call onChange value is from ngModel
            return;
        }
        this.onChange(this.value);
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        this.setOptionIfChanged('readOnly', this.disabled);
    }
    normalizeLineEndings(str) {
        return (str || '').replace(/\r\n|\r/g, '\n');
    }
    updateLineNumbers() {
        if (this.codeMirror) {
            this.codeMirror.setOption('lineNumbers', !!/\n/.test(this._value));
        }
    }
};
MatCodemirrorComponent.materialId = 0;
MatCodemirrorComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: KeyValueDiffers },
    { type: NgZone }
];
__decorate([
    HostBinding()
], MatCodemirrorComponent.prototype, "id", void 0);
__decorate([
    HostBinding('attr.aria-describedby')
], MatCodemirrorComponent.prototype, "describedBy", void 0);
__decorate([
    HostBinding('class.ngx-mat-codemirror')
], MatCodemirrorComponent.prototype, "ngxMatCodemirror", void 0);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "className", void 0);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "name", void 0);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "autoFocus", void 0);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "preserveScrollPosition", void 0);
__decorate([
    Output()
], MatCodemirrorComponent.prototype, "cursorActivity", void 0);
__decorate([
    Output()
], MatCodemirrorComponent.prototype, "focusChange", void 0);
__decorate([
    Output()
], MatCodemirrorComponent.prototype, "scroll", void 0);
__decorate([
    ViewChild('ref', { static: true })
], MatCodemirrorComponent.prototype, "ref", void 0);
__decorate([
    HostBinding('class.floating')
], MatCodemirrorComponent.prototype, "shouldLabelFloat", null);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "required", null);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "placeholder", null);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "disabled", null);
__decorate([
    Input()
], MatCodemirrorComponent.prototype, "options", null);
MatCodemirrorComponent = MatCodemirrorComponent_1 = __decorate([
    Component({
        selector: 'mat-codemirror',
        template: "<textarea\n  [name]=\"name\"\n  class=\"ngx-codemirror {{ className }}\"\n  [class.ngx-codemirror--focused]=\"focused\"\n  autocomplete=\"off\"\n  [autofocus]=\"autoFocus\"\n  #ref>\n</textarea>\n",
        providers: [
            {
                provide: MatFormFieldControl,
                useExisting: MatCodemirrorComponent_1
            }
        ],
        preserveWhitespaces: false,
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        styles: [".ngx-mat-codemirror .CodeMirror{height:auto}.ngx-mat-codemirror .CodeMirror-scroll{max-height:250px}"]
    }),
    __param(0, Optional()), __param(0, Self())
], MatCodemirrorComponent);

let MatCodemirrorModule = class MatCodemirrorModule {
};
MatCodemirrorModule = __decorate([
    NgModule({
        declarations: [
            MatCodemirrorComponent,
        ],
        exports: [
            MatCodemirrorComponent,
        ]
    })
], MatCodemirrorModule);

/**
 * Generated bundle index. Do not edit.
 */

export { MatCodemirrorModule, MatCodemirrorComponent as ɵa };
//# sourceMappingURL=ngx8-mat-codemirror.js.map
