import { __decorate, __param } from 'tslib';
import { EventEmitter, Optional, Self, KeyValueDiffers, NgZone, HostBinding, Input, Output, ViewChild, Component, ChangeDetectionStrategy, ViewEncapsulation, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { MatFormFieldControl } from '@angular/material';
import { Subject } from 'rxjs';

var MatCodemirrorComponent = /** @class */ (function () {
    function MatCodemirrorComponent(ngControl, // mat
    _differs, _ngZone) {
        this.ngControl = ngControl;
        this._differs = _differs;
        this._ngZone = _ngZone;
        this.stateChanges = new Subject();
        this.id = "mat-codemirror-component-" + MatCodemirrorComponent_1.materialId++;
        this.describedBy = '';
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this._required = false;
        this._focused = false;
        this._disabled = false;
        this._value = '';
        this.ngxMatCodemirror = true;
        /* class applied to the created textarea */
        this.className = '';
        /* name applied to the created textarea */
        this.name = 'codemirror';
        /* autofocus setting applied to the created textarea */
        this.autoFocus = false;
        /* preserve previous scroll position after updating value */
        this.preserveScrollPosition = false;
        /* called when the text cursor is moved */
        this.cursorActivity = new EventEmitter();
        /* called when the editor is focused or loses focus */
        this.focusChange = new EventEmitter();
        /* called when the editor is scrolled */
        this.scroll = new EventEmitter();
        if (this.ngControl != null) {
            this.ngControl.valueAccessor = this;
        }
    }
    MatCodemirrorComponent_1 = MatCodemirrorComponent;
    Object.defineProperty(MatCodemirrorComponent.prototype, "shouldLabelFloat", {
        get: function () {
            return this.focused || !this.empty;
        },
        enumerable: true,
        configurable: true
    });
    MatCodemirrorComponent.prototype.setDescribedByIds = function (ids) {
        this.describedBy = ids.join(' ');
    };
    MatCodemirrorComponent.prototype.onContainerClick = function (event) { };
    Object.defineProperty(MatCodemirrorComponent.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (req) {
            this._required = coerceBooleanProperty(req);
            this.stateChanges.next();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "errorState", {
        get: function () {
            return this.ngControl && !this.ngControl.pristine && !this.ngControl.valid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "placeholder", {
        get: function () {
            return this._placeholder;
        },
        set: function (plh) {
            this._placeholder = plh;
            this.stateChanges.next();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "empty", {
        get: function () {
            return !this.value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this._value = value;
            this.updateLineNumbers();
            this.stateChanges.next();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (dis) {
            this._disabled = coerceBooleanProperty(dis);
            this.stateChanges.next();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "focused", {
        get: function () {
            return this._focused;
        },
        set: function (val) {
            this._focused = val;
            this.stateChanges.next();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MatCodemirrorComponent.prototype, "options", {
        /**
         * set options for codemirror
         * @link http://codemirror.net/doc/manual.html#config
         */
        set: function (value) {
            this._options = value;
            if (!this._differ && value) {
                this._differ = this._differs.find(value).create();
            }
        },
        enumerable: true,
        configurable: true
    });
    MatCodemirrorComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (!this.ref) {
            return;
        }
        // in order to allow for universal rendering, we import Codemirror runtime with `require` to prevent node errors
        var fromTextArea = require('codemirror').fromTextArea;
        this.codeMirror = fromTextArea(this.ref.nativeElement, this._options);
        this.updateLineNumbers();
        this._ngZone.runOutsideAngular(function () {
            _this.codeMirror.on('change', _this.codemirrorValueChanged.bind(_this));
            _this.codeMirror.on('cursorActivity', _this.cursorActive.bind(_this));
            _this.codeMirror.on('focus', _this.focusChanged.bind(_this, true));
            _this.codeMirror.on('blur', _this.focusChanged.bind(_this, false));
            _this.codeMirror.on('scroll', _this.scrollChanged.bind(_this));
        });
        this.codeMirror.setValue(this.value);
    };
    MatCodemirrorComponent.prototype.ngDoCheck = function () {
        var _this = this;
        if (this._differ) {
            // check options have not changed
            var changes = this._differ.diff(this._options);
            if (changes) {
                changes.forEachChangedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
                changes.forEachAddedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
                changes.forEachRemovedItem(function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                });
            }
        }
    };
    MatCodemirrorComponent.prototype.ngOnDestroy = function () {
        // is there a lighter-weight way to remove the cm instance?
        if (this.codeMirror) {
            this.codeMirror.toTextArea();
        }
        this.stateChanges.complete();
    };
    MatCodemirrorComponent.prototype.codemirrorValueChanged = function (cm, change) {
        if (change.origin !== 'setValue') {
            this.value = cm.getValue();
            this.writeValue(cm.getValue());
        }
    };
    MatCodemirrorComponent.prototype.setOptionIfChanged = function (optionName, newValue) {
        if (!this.codeMirror) {
            return;
        }
        this.codeMirror.setOption(optionName, newValue);
    };
    MatCodemirrorComponent.prototype.focusChanged = function (focused) {
        this.onTouched();
        this.focused = focused;
        this.focusChange.emit(focused);
    };
    MatCodemirrorComponent.prototype.scrollChanged = function (cm) {
        this.scroll.emit(cm.getScrollInfo());
    };
    MatCodemirrorComponent.prototype.cursorActive = function (cm) {
        this.cursorActivity.emit(cm);
    };
    MatCodemirrorComponent.prototype.writeValue = function (value) {
        if (!this.codeMirror) {
            if (value) {
                this.value = value;
            }
            return;
        }
        if (value == null) {
            value = '';
        }
        if (value !== this.codeMirror.getValue() &&
            this.normalizeLineEndings(this.codeMirror.getValue()) !==
                this.normalizeLineEndings(value)) {
            this.value = value;
            if (this.preserveScrollPosition) {
                var prevScrollPosition = this.codeMirror.getScrollInfo();
                this.codeMirror.setValue(this.value);
                this.codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
                return;
            }
            this.codeMirror.setValue(this.value);
            // Don't call onChange value is from ngModel
            return;
        }
        this.onChange(this.value);
    };
    MatCodemirrorComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    MatCodemirrorComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    MatCodemirrorComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.setOptionIfChanged('readOnly', this.disabled);
    };
    MatCodemirrorComponent.prototype.normalizeLineEndings = function (str) {
        return (str || '').replace(/\r\n|\r/g, '\n');
    };
    MatCodemirrorComponent.prototype.updateLineNumbers = function () {
        if (this.codeMirror) {
            this.codeMirror.setOption('lineNumbers', !!/\n/.test(this._value));
        }
    };
    var MatCodemirrorComponent_1;
    MatCodemirrorComponent.materialId = 0;
    MatCodemirrorComponent.ctorParameters = function () { return [
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: KeyValueDiffers },
        { type: NgZone }
    ]; };
    __decorate([
        HostBinding()
    ], MatCodemirrorComponent.prototype, "id", void 0);
    __decorate([
        HostBinding('attr.aria-describedby')
    ], MatCodemirrorComponent.prototype, "describedBy", void 0);
    __decorate([
        HostBinding('class.ngx-mat-codemirror')
    ], MatCodemirrorComponent.prototype, "ngxMatCodemirror", void 0);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "className", void 0);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "name", void 0);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "autoFocus", void 0);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "preserveScrollPosition", void 0);
    __decorate([
        Output()
    ], MatCodemirrorComponent.prototype, "cursorActivity", void 0);
    __decorate([
        Output()
    ], MatCodemirrorComponent.prototype, "focusChange", void 0);
    __decorate([
        Output()
    ], MatCodemirrorComponent.prototype, "scroll", void 0);
    __decorate([
        ViewChild('ref', { static: true })
    ], MatCodemirrorComponent.prototype, "ref", void 0);
    __decorate([
        HostBinding('class.floating')
    ], MatCodemirrorComponent.prototype, "shouldLabelFloat", null);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "required", null);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "placeholder", null);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "disabled", null);
    __decorate([
        Input()
    ], MatCodemirrorComponent.prototype, "options", null);
    MatCodemirrorComponent = MatCodemirrorComponent_1 = __decorate([
        Component({
            selector: 'mat-codemirror',
            template: "<textarea\n  [name]=\"name\"\n  class=\"ngx-codemirror {{ className }}\"\n  [class.ngx-codemirror--focused]=\"focused\"\n  autocomplete=\"off\"\n  [autofocus]=\"autoFocus\"\n  #ref>\n</textarea>\n",
            providers: [
                {
                    provide: MatFormFieldControl,
                    useExisting: MatCodemirrorComponent_1
                }
            ],
            preserveWhitespaces: false,
            changeDetection: ChangeDetectionStrategy.OnPush,
            encapsulation: ViewEncapsulation.None,
            styles: [".ngx-mat-codemirror .CodeMirror{height:auto}.ngx-mat-codemirror .CodeMirror-scroll{max-height:250px}"]
        }),
        __param(0, Optional()), __param(0, Self())
    ], MatCodemirrorComponent);
    return MatCodemirrorComponent;
}());

var MatCodemirrorModule = /** @class */ (function () {
    function MatCodemirrorModule() {
    }
    MatCodemirrorModule = __decorate([
        NgModule({
            declarations: [
                MatCodemirrorComponent,
            ],
            exports: [
                MatCodemirrorComponent,
            ]
        })
    ], MatCodemirrorModule);
    return MatCodemirrorModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { MatCodemirrorModule, MatCodemirrorComponent as ɵa };
//# sourceMappingURL=ngx8-mat-codemirror.js.map
